#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Core {
  /// <summary>
  /// Represents the base class for all basic item types.
  /// </summary>
  [StorableClass]
  [Item("Item", "Base class for all HeuristicLab items.")]
  public abstract class Item : IItem {
    private string itemName;
    public virtual string ItemName {
      get {
        if (itemName == null) {
          itemName = ItemAttribute.GetName(this.GetType());
        }
        return itemName;
      }
    }
    private string itemDescription;
    public virtual string ItemDescription {
      get {
        if (itemName == null) {
          itemName = ItemAttribute.GetDescription(this.GetType());
        }
        return itemName;
      }
    }
    private Version itemVersion;
    public Version ItemVersion {
      get {
        if (itemVersion == null) {
          itemVersion = ItemAttribute.GetVersion(this.GetType());
        }
        return itemVersion;
      }
    }
    public static Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Class; }
    }
    private Image itemImage;
    public virtual Image ItemImage {
      get {
        if (itemImage == null) {
          itemImage = ItemAttribute.GetImage(this.GetType());
        }
        return itemImage;
      }
    }

    [StorableConstructor]
    protected Item(bool deserializing) { }
    protected Item(Item original, Cloner cloner) {
      cloner.RegisterClonedObject(original, this);
    }
    protected Item() { }

    public object Clone() {
      return Clone(new Cloner());
    }
    public abstract IDeepCloneable Clone(Cloner cloner);

    /// <summary>
    /// Gets the string representation of the current instance.
    /// </summary>
    /// <returns>The type name of the current instance.</returns>
    public override string ToString() {
      return ItemName;
    }

    public event EventHandler ItemImageChanged;
    protected virtual void OnItemImageChanged() {
      EventHandler handler = ItemImageChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
    public event EventHandler ToStringChanged;
    protected virtual void OnToStringChanged() {
      EventHandler handler = ToStringChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
  }
}
